#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/mount.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <dirent.h>

#include <linux/loop.h>
#include <errno.h>

#ifndef TIMESTAMP
  #define TIMESTAMP "no timestamp set, probably a test build"
#endif

int main()
{
// printf("Searching for %s...\n", TIMESTAMP);
  mount("", "/dev", "devtmpfs", 0, 0);
  // Read kernel command arguments
  mount("", "/proc", "proc", 0, 0);
  int i=open("/proc/cmdline", O_RDONLY);
  int cmdlen=128;
  char* cmdline=malloc(sizeof(char)*cmdlen);
  int len;
  // Note: we can't just stat /proc/cmdline to get the size because it's a virtual file
  while((len=read(i, &cmdline[cmdlen-128], 128))==128)
  {
    cmdlen+=128;
    cmdline=realloc(cmdline, sizeof(char)*cmdlen);
  }
  cmdline[cmdlen-128+len]=0;
  close(i);
  umount("/proc");
  rmdir("/proc");
  char quiet=0;
  if(strstr(cmdline, " quiet ")||strstr(cmdline, " quiet\n")){quiet=1;}
  char* initexec=0;
  if((initexec=strstr(cmdline, " init=")))
  {
    initexec=&initexec[6];
    for(i=0; initexec[i]&&initexec[i]!=' '&&initexec[i]!='\n'; ++i);
    initexec[i]=0;
  }
  if(!initexec){initexec="/sbin/init";}
  free(cmdline); // not needed anymore

  // Potential devices where we may find the AOS live system
  char* devices[]={"sr0","sr1","sr2","sr3","sda","sdb","sdc","sdd","sde","sdf","sdg","sdh","hda","hdb","hdc","hdd","hde","hdf","hdg","hdh"};
  int devicenum=sizeof(devices)/sizeof(char*);
  char path[]="/dev/xxx";
  while(1)
  {
    for(i=0; i<devicenum; ++i)
    {
      if(!quiet) printf("Checking if /dev/%s is our liveCD... ", devices[i]);
      memcpy(&path[5], devices[i], 3);
      if(mount(path, "/.cdrom", "iso9660", MS_RDONLY, 0))
      {
        if(!quiet) printf("no\n");
      }
      // check that /boot/live-initrd exists so we know it's not some other CD. Timestamp to avoid any mixups with similar discs
      else if(access("/.cdrom/boot/live-initrd-" TIMESTAMP, F_OK))
      {
        if(!quiet) printf("no\n");
      }else{
        char loopfile[16];
        int loopcontrol=open("/dev/loop-control", O_RDWR);
        int loopnr=ioctl(loopcontrol, LOOP_CTL_GET_FREE);
        sprintf(loopfile, "/dev/loop%i", loopnr);
        int imagefile=open("/.cdrom/root.squash", O_RDONLY);
        int loopdev=open(loopfile, O_RDWR);
        if(ioctl(loopdev, LOOP_SET_FD, imagefile)){perror("ioctl(LOOP_SET_FD)");}
//        if(mount("/.cdrom/root.squash", "/.cdrom", "squashfs", MS_RDONLY, 0))
        if(mount(loopfile, "/.cdrom", "squashfs", MS_RDONLY, 0)){perror("mount"); sleep(10);}
        if(!quiet) printf("yes");
        // Clear out the ramdisk content
        umount("/dev");
        rmdir("/dev");
        unlink("/init");
        // Move our compressed filesystem to root
        chdir("/.cdrom");
        mount(".", "/", 0, MS_MOVE, 0);
        chroot(".");
        mount("", "/dev", "devtmpfs", 0, 0); // Re-mount /dev in new root
        if(!quiet) printf("\n");
        execl(initexec, initexec, (void*)0);
        return 1;
      }
    }
    if(!quiet) printf("Waiting for root to become available...");
    sleep(3);
    if(!quiet) printf("\n");
  }
  return 0;
}
